use gtk4 :: Image ;
use gtk4 :: Align :: * ;
use libadwaita :: * ;
use std :: cell :: * ;

use crate :: utility :: * ;
use crate :: ui :: * ;
use crate :: ui :: utility :: * ;
use crate :: ui :: collatable_container :: SortingDirection :: * ;



pub struct MediaSortButton < A : MediaAdapter > {
	widget : SplitButton ,
	previous_sorting : & 'static RefCell < A :: Sorting > ,
}

impl < A : MediaAdapter > MediaSortButton <A> {
	pub fn new ( on_sort : impl Fn ( A :: Sorting ) + 'static ) -> Self {
		let previous_sorting = leak ( RefCell :: new ( A :: Sorting :: default () ) ) ;
		let property_descriptions = A :: get_property_descriptions () ;

		let sort_icons = {
			let mut sort_icons = Vec :: new () ;
			for _ in property_descriptions {
				sort_icons . push ( image ! ( @ icon_name : "view-sort-ascending-symbolic" ; ) ) ;
			}
			Box :: leak ( sort_icons . into_boxed_slice () ) as & 'static _
		} ;

		let list_box = list_box ! (
			@ connect_row_activated : move | _ , row | on_media_sort_activated :: <A> (
				row . index () ,
				previous_sorting ,
				& on_sort ,
				sort_icons ,
			) ;
		) ;
		for ( index , ( _ , description ) ) in property_descriptions . iter () . enumerate () {
			list_box . append ( & g_box ! (
				@ orientation : Horizontal ; @ spacing : 20 ;
				& label ! ( @ hexpand : true ; @ halign : Start ; description ) ,
				& sort_icons [index] ,
			) ) ;
		}

		let widget = split_button ! (
			@ popover : & popover ! ( @ css_classes : & [ "menu" ] ; & list_box ) ;
			& label ! ("Sort") ,
		) ;

		Self { widget , previous_sorting }
	}
}

impl < A : MediaAdapter > Component for MediaSortButton <A> {
	fn get_widget ( & self ) -> & SplitButton { & self . widget }
}

fn on_media_sort_activated < A : MediaAdapter > (
	row : i32 ,
	previous_sorting_mut : & RefCell < A :: Sorting > ,
	on_sort : & impl Fn ( A :: Sorting ) ,
	sort_icons : & [ Image ] ,
) {
	let row = row as usize ;
	debug_assert ! (
		row <= A :: get_property_descriptions () . len () ,
		"Sorting menu has more rows than media adapter has property descriptions" ,
	) ;
	let ( sorting_property , _ ) = A :: get_property_descriptions () [row] . clone () ;

	let previous_sorting = * previous_sorting_mut . borrow () ;
	if sorting_property == previous_sorting . get_property () {
		match previous_sorting . get_direction () {
			Ascending => {
				let new_sorting = A :: Sorting :: new ( sorting_property , Descending ) ;
				previous_sorting_mut . replace (new_sorting) ;
				sort_icons [row] . set_icon_name ( Some ("view-sort-descending-symbolic") ) ;
				on_sort (new_sorting) ;
			} ,
			Descending => {
				let new_sorting = A :: Sorting :: new ( sorting_property , Ascending ) ;
				previous_sorting_mut . replace (new_sorting) ;
				sort_icons [row] . set_icon_name ( Some ("view-sort-ascending-symbolic") ) ;
				on_sort (new_sorting) ;
			} ,
		}
	} else {
		let new_sorting = A :: Sorting :: new ( sorting_property , Ascending ) ;
		previous_sorting_mut . replace (new_sorting) ;
		sort_icons [row] . set_icon_name ( Some ("view-sort-ascending-symbolic") ) ;
		on_sort (new_sorting) ;
	}
}
